import {XMLParser, type X2jOptions} from 'fast-xml-parser';
import { UAObject } from './UAObject';
import type { UABaseNode } from './UABaseNode';
import { UAVariable } from './UAVariable';
import { NamespaceTable } from './NameSpaceTable';
import { Model } from './Model';
import { XMLElem, type IToXML } from '@/util/XmlElem';

export class UANodeSet implements IToXML{

    constructor(public fileName: string,
                public models: Model[],
                public nodes: UABaseNode[],
                public nameSpaceTable: NamespaceTable) {
    }

    reIndex(nst: NamespaceTable) {
        //add all missing namespaces to addressspace ns table
        for(const value of this.nameSpaceTable.nsMap.getValues()) {
            nst.addUri(value);
        }
        for(const node of this.nodes) {
            node.reIndex(nst, this.nameSpaceTable);
        }
    }

    resolveChildren(nm: Map<string, UABaseNode>) {
        for(const node of this.nodes) {
            node.resolveChildren(nm);
        }    
    }

    toXML(): XMLElem {
        const xmlUANodeSet =new XMLElem('UANodeSet');
        const xmlModels=xmlUANodeSet.add(new XMLElem('Models'));
        for(const model of this.models) {
            xmlModels.add(model.toXML())
        }
        for(const node of this.nodes) {
            xmlUANodeSet.add(node.toXML())
        }
        return xmlUANodeSet;
    }

    static async load(url: string): Promise<UANodeSet> {
        const xml= await (await fetch(url)).text();
        const fileName= url.split('/').pop()||url
        return this.parse(xml, fileName);
    }
    static async parse(xml: string, fileName: string): Promise<UANodeSet> {
        const parseOptions:Partial<X2jOptions>={
            ignoreAttributes: false,
            numberParseOptions: {
                hex: false,
                leadingZeros: false,
                skipLike: /./
            },
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            isArray: (name, jpath, isLeafNode, isAttribute):boolean => { 
                switch(jpath) {
                    case 'UANodeSet.NamespaceUris.Uri':
                    case 'UANodeSet.UAObject.References.Reference':
                    case 'UANodeSet.UAObject':
                    case 'UANodeSet.UAVariable':
                    case 'UANodeSet.UAVariable.References.Reference':
                    case 'UANodeSet.Models.Model':
                        return true;
                    default:
                        return false;
                }
            }
        }
        const parser = new XMLParser(parseOptions);
        const xmlObj = parser.parse(xml);
        const models: Model[]=[];
        const xmlModels=xmlObj['UANodeSet']['Models'];
        for(const xmlModel of xmlModels.Model) {
            models.push(Model.fromXML(xmlModel));
        }
        const nodes:UABaseNode[]=[];
        const xmlObjects=xmlObj['UANodeSet']['UAObject'];
        for(const xmlObject of xmlObjects) {
            nodes.push(UAObject.fromXML(xmlObject));
        }
        const xmlVariables=xmlObj['UANodeSet']['UAVariable'];
        for(const xmlVariable of xmlVariables) {
            nodes.push(UAVariable.fromXML(xmlVariable));
        }
        const uaNamespaceUris=xmlObj['UANodeSet']['NamespaceUris'];
        const nst=new NamespaceTable();
        if(uaNamespaceUris) {
            for(const nsUri of uaNamespaceUris['Uri']) {
                nst.addUri(nsUri)
            }
        }
        return new UANodeSet(fileName, models, nodes, nst);
    }
}